@@ -18,34 +18,43 @@
 
   2002/11/15 - Updated by Kostya Kortchinsky <kostya.kortchinsky@renater.fr>
   - ported to SunOS.
-  
+
   2003/01/19 - Another Adore based lkm test. Original idea from Junichi Murakami
 
   2003/02/02 - More little fixes - Nelson Murilo
 
   2003/02/23 - Use of kill to eliminate false-positives abandonated, It is
-  preferable false-positives that false-negatives. Uncomment kill() functions 
-  if you like  it. 
+  preferable false-positives that false-negatives. Uncomment kill() functions
+  if you like  it.
 
-  2003/06/07 - Fix for NPTL threading mechanisms - patch by Mike Griego 
+  2003/06/07 - Fix for NPTL threading mechanisms - patch by Mike Griego
+
+  2003/09/01 - Fix for ps mode detect, patch by Bill Dupree and others
+
+  2004/04/03 - More fix for linux's threads - Nelson Murilo
 
-  2003/09/01 - Fix for ps mode detect, patch by Bill Dupree and others 
- 
 */
 
 #if !defined(__linux__) && !defined(__FreeBSD__) && !defined(__sun)
-main (){ return 0; }
+int main (){ return 0; }
 #else
 #include <stdio.h>
-#include <errno.h> 
+#include <errno.h>
 #include <sys/types.h>
 #include <dirent.h>
+#include <ctype.h>
+#include <stdlib.h>
+#include <unistd.h>
+#include <signal.h>
 #if defined(__sun)
 #include <procfs.h>
 #include <fcntl.h>
 #endif
 
-//#define MAX_PROCESSES 65535
+#define PS_SUN 0
+#define PS_COM 1
+#define PS_LNX 2
+#define PS_MAX 2
 
 #if defined(__sun)
 #define FIRST_PROCESS 0
@@ -54,12 +63,12 @@ main (){ return 0; }
 #endif
 #define MAX_PROCESSES 99999
 #define MAX_BUF 1024
-#if defined(__sun)
-#define PS "ps -edf"
-#else
-#define PS  "ps mauxwww 2>/dev/null"
-#endif
-#define PS1 "ps auxwww"
+
+static char *ps_cmds[] = {
+	    "ps -edf",
+	    "ps auxw",
+            "ps auxw -T|tr -s ' '|cut -d' ' -f2-",
+          };
 
 int psproc [MAX_PROCESSES+1];
 int dirproc[MAX_PROCESSES+1];
@@ -67,15 +76,15 @@ int dirproc[MAX_PROCESSES+1];
 int isathread[MAX_PROCESSES+1];
 #endif
 
-main(int argc, char **argv)
+int main(int argc, char **argv)
 {
    char buf[MAX_BUF], *p, path[MAX_BUF];
-   FILE *ps  = popen(PS, "r");
+   char *pscmd = (char *)0;
+   FILE *ps;
    DIR *proc = opendir("/proc");
    struct dirent *dir;
-   int i, j, retps, retdir;
-   int verbose = 0;
-   long ret = 0;
+   int i, j, retps, retdir, pv, verbose;
+   long ret = 0L;
    char * tmp_d_name;
 #if defined(__linux__)
    int maybeathread;
@@ -84,12 +93,13 @@ main(int argc, char **argv)
    psinfo_t psbuf;
 #endif
 
-   if (!ps || !proc)
+   pv = verbose = 0; 
+
+   if (!proc)
    {
-      perror("ps or proc");
+      perror("proc");
       exit (1);
    }
-
    for (i = 1; i < argc; i++)
    {
       if (!memcmp(argv[i], "-v", 2))
@@ -99,7 +109,36 @@ main(int argc, char **argv)
 	printf("Usage: %s [-v] [-v]\n", argv[0]);
 	return 0;
       }
+#if defined(__linux__)
+      else if (!memcmp(argv[i], "-p", 2))
+        if (i+1 < argc) 
+           pv = atoi(argv[++i]);
+        else
+        {
+	   printf("Usage: %s [-v] [-v] [-p procps version]\n", argv[0]);
+	   return 0;
+        }
+#endif
    }
+#if defined(__sun)
+   pscmd = ps_cmds[PS_SUN];
+#elif !defined (__linux__)
+   pscmd = ps_cmds[PS_COM];
+#endif
+#if defined(__linux__)
+   if (pv < 1 || pv > PS_MAX)
+      pv = 1;
+   pscmd = ps_cmds[pv];
+/*  printf("pv = %d\n\r", pv); /* -- DEBUG */
+#endif
+
+/* printf("pscmd = %s\n\r", pscmd); /* -- DEBUG */ 
+   if (!(ps = popen(pscmd, "r")))
+   {
+       perror("ps");
+       exit(errno);
+   }
+
    *buf = 0;
    fgets(buf, MAX_BUF, ps); /* Skip header */
 #if defined(__sun)
@@ -108,15 +147,8 @@ main(int argc, char **argv)
    if (!isalpha(*buf))
 #endif
    {
-     fclose(ps); 
-     ps  = popen(PS1, "r");
-     if (!ps) 
-     {
-        perror("ps"); 
-        exit(1);
-     }
      fgets(buf, MAX_BUF, ps); /* Skip header */
-     if (!isalpha(*buf))
+     if (!isalpha(*buf) && pv != PS_LNX)
      {
         fprintf(stderr, "OooPS!\n");
         exit(2);
@@ -133,13 +165,14 @@ main(int argc, char **argv)
    {
       p = buf;
 #if defined(__sun)
-      while (*p == ' ') /* Skip spaces */
+      while (isspace(*p)) /* Skip spaces */
           p++;
 #endif
-      while (*p != ' ') /* Skip USER */
+      while (!isspace(*p)) /* Skip User */
           p++;
-      while (*p == ' ') /* Skip spaces */
+      while (isspace(*p)) /* Skip spaces */
           p++;
+/*  printf(">>%s<<\n", p);  /* -- DEBUG */
       ret = atol(p);
       if ( ret < 0 || ret > MAX_PROCESSES )
       {
@@ -150,7 +183,7 @@ main(int argc, char **argv)
    }
    pclose(ps);
 
-   while (dir = readdir(proc))
+   while ((dir = readdir(proc)))
    {
 #if defined(__linux__)
       maybeathread = 0;
@@ -174,6 +207,7 @@ main(int argc, char **argv)
       }
 #endif
 
+/*      printf("%s\n", tmp_d_name); /* -- DEBUG */
       dirproc[atol(tmp_d_name)] = 1;
    }
    closedir(proc);
@@ -186,19 +220,20 @@ main(int argc, char **argv)
       snprintf(&buf[6], 6, "%d", i);
       if (!chdir(buf))
       {
-         if (!dirproc[i] ) // && !kill(i, 0))
+/*         if (!dirproc[i] ) /*  && !kill(i, 0)) */
+         if (!dirproc[i] && !psproc[i])
          {
 #if defined(__linux__)
             if (!isathread[i]) {
 #endif
             retdir++;
             if (verbose)
-	       printf ("PID %5d: not in readdir output\n", i);
+	       printf ("PID %5d(%s): not in readdir output\n", i, buf);
 #if defined(__linux__)
             }
 #endif
          }
-         if (!psproc[i] ) // && !kill(i, 0))
+         if (!psproc[i] ) /* && !kill(i, 0)) */
          {
 #if defined(__linux__)
             if(!isathread[i]) {
@@ -213,7 +248,8 @@ main(int argc, char **argv)
 #if defined(__linux__)
          if(!isathread[i]) {
 #endif
-	 if ((!dirproc[i] || !psproc[i]) && !kill(i, 0) && (verbose > 1))
+/*	 if ((!dirproc[i] || !psproc[i]) && !kill(i, 0) && (verbose > 1)) */
+	 if ((!dirproc[i] || !psproc[i]) && (verbose > 1))
 	 {
 #if defined(__linux__)
 	    j = readlink ("./cwd", path, sizeof(path));
@@ -248,12 +284,12 @@ main(int argc, char **argv)
       printf("You have % 5d process hidden for readdir command\n", retdir);
    if (retps)
       printf("You have % 5d process hidden for ps command\n", retps);
-   kill(1, 100); //  Check for SIGINVISIBLE Adore signal
-   if ( errno == 3) 
+   kill(1, 100); /*  Check for SIGINVISIBLE Adore signal */
+   if ( errno == 3)
    {
-      printf("SIGINVISIBLE Adore found\n"); 
+      printf("SIGINVISIBLE Adore found\n");
       retdir+= errno;
-   } 
+   }
    return (retdir+retps);
 }
 #endif
