@@ -13,6 +13,8 @@
    Andre Gustavo <gustavo@anita.visualnet.com.br>
    12/15/00 - Add -f option
    Nelson Murilo, nelson@pangeia.com.br
+   07/08/04 - fix del counter value (Thanks to Dietrich Raisin)
+   Nelson Murilo, nelson@pangeia.com.br
 */
 
 
@@ -54,10 +56,10 @@ int main(int argc, char*argv[]) {
 	struct timeval	mytime;
 	struct timezone	dummy;
 	long		start_time, act_time;
-	int		del_counter;
+	int		del_counter, t_del;
         char wtmpfile[128];
 
-	del_counter=0;
+	del_counter=t_del=0;
 	start_time=0;
 
 	gettimeofday(&mytime, &dummy);
@@ -78,6 +80,7 @@ int main(int argc, char*argv[]) {
 			if (del_counter) {
 				printit(del_counter, start_time,
 					utmp_ent.ut_time);
+				t_del++;
 				del_counter=0;
 			}
 			start_time=utmp_ent.ut_time;
@@ -86,5 +89,5 @@ int main(int argc, char*argv[]) {
 	close(filehandle);
 	if (del_counter)
 	   printit(del_counter, start_time, act_time);
-        exit(del_counter);
+        exit(t_del+del_counter);
 }
